pro get_tot_cp,mode,y_input,m_input,d_input,no2_corr,pressure_corr,chan,leff_mode,year_count,time,am,rel_press,ozone,aot_mode,aeronet_aot,ifit,cimel_chan,l0_fit,l1_fit,l2_fit,v0_corr,$
cimel, v0_corr_cimel,w_vac_chen,srf,err_srf,solv_w,beta_w,alpha_w, tr_eff,ta_eff,toz_eff,tot_eff,l_eff,lam_dir,$
aer340cor,aer380cor,aer440cor,aer500cor, delta_aot_fit,sigma_aot_fit

;	PURPOSE
; calculate effective total extinction optical thickness for specific UV-MFRSR measurement

;	WARNING:
;	04/25/06:  if aot_mode='brewer' only dynamic mode, linear extrapolation
;	MODIFIED
;	04/25/06 - add Brewer AOT interpolation (linear lsf)                                                                
;	05/20/05 - add 3 wavelength Lagrange interpolation (cimel_chan=3) as opposed to 4 wavelength lsf fit (cimel_chan=4)
;	03/07/05 - V0 correction


;	INPUT
; aot_mode      i  char: 'brewer' or 'aeronet' depending on AOT source for calibration
; brewer_wav(0,1,2,3,4,5) i  Brewer 6 wavelengths for specific slit mask
; brewer_aot(0,..,5)	i Brewer  AOT for specific slit mask  (uncorrected ! )
; mode eq'screen" or 'print' for plot_aot_spec subroutine
; pressure_corr i lolgical Whether to apply pressure correction to AERONET AOT data
; chan 		i character MFRSR spectral channel (car)=['300nm','305nm', ...,'368nm']
; leff_mode	i eq 'staic'  use pre-calculated tau_eff for each MFRSR spectral channel (w_vac_mfrsr)
;           	i eq 'dynamic' use dynamic calculation of l_eff and tau_eff for each measurement
; w_vac_chen    i all wavelengths(vac) for the chan   (used only in "dynamic" mode)
; year_count 	i date of measurement=year+doy/365.25
; time		i time 
; am		i airmass 
; rel_press	i P/P0
; ozone		i ozone [DU]
;aeronet_aot(0,1,2,3)	i Aeronet AOT at 340, 380,440,500 (uncorrected ! )
; ifit          i AERONET spectral extrapolation method (1- linear, 2- quadratic)
; 			(ifit=1  ; linear interpolation/extrapolation lntau vs ln wavelength
;  			(ifit=2  ; quadratic least square regression of  lntau vs ln wavelength
; 			ln(w_vac_mfrsr) if leff_mode eq 'static'   OR
;			alog(w_vac_chen) if  leff_mode eq 'dynamic'
; cimel_chan	i number of AERONET channels in quadratic interpolation
;  		 cimel_chan=4: 2nd order  least squares fit of aeronet ln(tau_aer(340,380,440nm,500nm)) to alog(w_vac_chen)
;  		 cimel_chan=3: Lagrangial  interpolation using just 3 channels: l0_fit,l1_fit,l2_fit    to alog(w_vac_chen)
;							
; l0_fit	i 1st wavelength index in linear fit (ifit=1)  [ 0 ,1  ,2  ,3]
; l1_fit	i 2nd wavelength index in linear fit (ifit=1)  [340,380,440,500]
; l2_fit	i 3d  wavelength index in quadratic  (ifit=2, cimel_chan=3) 
;       	i All 4 CIMEL wavelengths are used for lsf quadratic fit if cimel_chan=4 !
; v0_corr	l  'yes' or 'no'  to perform V0 correction
; 

;	OUTPUT
; cimel		o   master cimel instrument number
; v0_corr_cimel o   CIMEL specific V0 corrections (derived from ML calibrations)
; tr_eff 	o  tau_ray at lam_dir for a given channel (chan)
; ta_eff 	o  AOT    at lam_dir for a given channel (chan) and all corrections to AERONET AOT (pressure, V0, NO2, etc)
; toz_eff 	o  tau_oz at lam_dir for a given channel (chan)
; l_eff 	o  band effective wavelength (vac) for channel
; lam_dir 	o  band direct radiance equivalent wavelength (vac) for channel
; aer340cor 	o corrcted aeronet_aot(0)  (for pressure, V0 , NO2, etc) referred to central wavelength (l_cent) of CIMEL 340nm filter
; aer380cor	o  corrcted aeronet_aot(0)  (for pressure, V0 , NO2, etc) referred to central wavelength (l_cent) of CIMEL 380nm filter
; aer440cor	o  corrcted aeronet_aot(0)  (for pressure, V0 , NO2, etc) referred to central wavelength (l_cent) of CIMEL 440nm filter
; aer500cor	o  corrcted aeronet_aot(0)  (for pressure, V0 , NO2, etc) referred to central wavelength (l_cent) of CIMEL 500nm filter
; delta_aot_fit o difference between linear and quadratic fit
; sigma_aot_fit o standard deviation of AOT quadratic fit

;print,'Aerosol: ifit=',ifit
nw_vac=n_elements(w_vac_chen)

IF leff_mode eq 'static' THEN BEGIN             ;&    print,'Static wavelength correction '

get_xeff,w_vac_mfrsr,beta_ray,alpha_45    ; re-calculate MFRSR vac wavelengths and ETS average molecular cross-sections (vac)
ln_wav_mfrsr=alog(w_vac_mfrsr) & ;print,'Wavelength' &  print,w_vac_mfrsr

aeronet_aot_ext,mode,y_input,m_input,d_input,time,am,year_count,pressure_corr,rel_press,ifit,cimel_chan,l0_fit,l1_fit,l2_fit,no2_corr,cimel,aeronet_aot,ln_wav_mfrsr,aeronet_aot_mfrsr,$
aer340cor,aer380cor,aer440cor,aer500cor,delta_aot_fit,sigma_aot_fit


if chan eq '300nm' then begin
ta_eff=aeronet_aot_mfrsr(0) 
end else if chan eq '305nm' then begin
ta_eff=aeronet_aot_mfrsr(1) 
end else if chan eq '311nm' then begin
ta_eff=aeronet_aot_mfrsr(2) 
end else if chan eq '317nm' then begin
ta_eff=aeronet_aot_mfrsr(3) 
end else if chan eq '325nm' then begin
ta_eff=aeronet_aot_mfrsr(4) 
end else if chan eq '332nm' then begin
ta_eff=aeronet_aot_mfrsr(5) 
end else if chan eq '368nm' then begin
ta_eff=aeronet_aot_mfrsr(6) 
end else begin
;print,'get_tot_cp> not valid channel: chan=',chan
stop
end
tr_eff= beta_ray(6)*rel_press  
toz_eff=ozone*0.001*alpha_45(6) &  tot_eff=ta_eff + tr_eff + toz_eff            
l_eff=368. & l_rad=368.


ENDIF  ELSE IF  leff_mode eq 'dynamic' THEN BEGIN	; correction for direct irradiance equivalent wavelength

;print,'Dynamic wavelength correction ' &  print,'Wavelength' & print,w_vac_chen
e0=int_tabulated(w_vac_chen,solv_w*srf) 		;& print,'e0=',e0

tr_chen=rel_press*beta_w    	; Rayleigh at srf vac wavelengths
;print,'Rayleigh' & print,tr_chen

toz_chen=0.001*ozone*alpha_w	; Ozone at srf vac wavelengths
;print,'Ozone'   & print,toz_chen

;print,'get_tot_cp: year_count=',year_count
;aeronet_aot_ext,mode,y_input,m_input,d_input,time,am,year_count,pressure_corr,rel_press,ifit,cimel_chan,l0_fit,l1_fit,l2_fit,no2_corr,v0_corr,cimel,v0_corr_cimel,aeronet_aot,brewer_aot,alog(w_vac_chen),ta_chen,$
;aer340cor,aer380cor,aer440cor,aer500cor,delta_aot_fit,sigma_aot_fit
aot_ext,mode,y_input,m_input,d_input,time,am,year_count,pressure_corr,rel_press,ifit,cimel_chan,l0_fit,l1_fit,l2_fit,no2_corr,v0_corr,cimel,v0_corr_cimel,aot_mode,aeronet_aot,alog(w_vac_chen),ta_chen,$
aer340cor,aer380cor,aer440cor,aer500cor,delta_aot_fit,sigma_aot_fit

print,'get_tot_cp: spectral Aerosol'  & print,ta_chen
;print,'Ozone=',ozone
;print,'Pressure=',rel_press

t_spec=exp(-am*(tr_chen + ta_chen +toz_chen))        ; calculated spectral transmittance
;print,'t_spec' & print,t_spec
e_bot=int_tabulated(w_vac_chen,t_spec*solv_w*srf)  ;& print,'e_bot=',e_bot
t_meas=double(e_bot/e0)         ; "Measured  bandpass average transmittance"
tot_eff=-alog(t_meas)/am
;print,'tot_eff',tot_eff

;       Flux weighted (effective) wavelength
l_eff=int_tabulated(w_vac_chen,w_vac_chen*t_spec*solv_w*srf)/e_bot
;;print,'L_eff=',l_eff
; Radiative equivalent wavelength:
l_rad=interpol(double(w_vac_chen),double(t_spec),double(t_meas))
if  abs(l_rad(0)-l_eff) gt 0.5  then begin      ; check closeness to l_eff
;print,'!!! l_rad too far from l_eff !!!'  & print,l_rad(0),l_eff
	for i=1,nw_vac-5 do begin       ; start removing shorter wavelengths iteratively
	l_rad=interpol(double(w_vac_chen(i:nw_vac-1)),double(t_spec(i:nw_vac-1)),double(t_meas)) ;& print,'l_rad,l_eff',i,l_rad(0),l_eff
	if  abs(l_rad(0)-l_eff) le 0.5  then goto,quit  ; quit check l_eff closiness
endfor
print,' !!! Fails to find L_rad: L_rad=L_eff  '
l_rad(0)=l_eff
endif
quit:           ; END L_eff calculation
lam_dir=l_rad(0)  ; & print,'l_rad=',lam_dir 

;	Effective tau at lam_dir
tr_eff=interpol(tr_chen,w_vac_chen,lam_dir) ;&  print,'get_tot_cp: tr_eff=',tr_eff
if tr_eff(0) gt max(tr_chen) or tr_eff(0) lt min(tr_chen) then begin & print,'        !!! tr_eff outside boundaries: ' & print,tr_eff(0),min(tr_chen),max(tr_chen)  & stop & endif

print,'lam_dir=',lam_dir
;print,'w_vac_chen=',w_vac_chen

ta_eff=interpol(ta_chen,w_vac_chen,lam_dir) &  print,'get_tot_cp: ta_eff=',ta_eff(0)
if ta_eff(0) gt max(ta_chen) or ta_eff(0) lt min(ta_chen) then begin & print,'        !!! ta_eff(0) outside boundaries !!!' & stop & endif

toz_eff =interpol(toz_chen,w_vac_chen,lam_dir) ;&  print,'toz_eff=',toz_eff(0)
if toz_eff(0) lt min(toz_chen) or toz_eff(0) gt max(toz_chen) then begin & print,'    !!! tau_oz_eff(0) outside boundaries !!!' & stop & endif

;print,'toz_eff+ta_eff+tr_eff',toz_eff+ta_eff+tr_eff

END		; end dynamic tau calculation

return
end
